package fr.username404.snowygui.gui.elements

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.DefaultVertexFormat
import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.Renderable.Rendering.buffer
import fr.username404.snowygui.gui.Renderable.Rendering.defaultRectFunc
import fr.username404.snowygui.gui.Renderable.Rendering.tessellator
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import net.minecraft.client.Minecraft
import net.minecraft.network.chat.TranslatableComponent
import org.lwjgl.opengl.GL20

class ClickBox(
    x: Double, y: Double,
    private val name: TranslatableComponent? = null,
    val buttons: MutableCollection<ClickButton> = mutableListOf()
): ColoredElement(x, y, 80, 10, 0x6C9E9D, 0.5F) {
    @JvmField
    val scrollBar: ColoredElement = object: ColoredElement(
        (x + width), y + height + 3,
        color = color, width = 3, height = 74,
        opacity = 0.75F
    ) {
        override fun render(poseStack: PoseStack?) {
            RenderSystem.disableTexture()
            RenderSystem.enableBlend()
            RenderSystem.defaultBlendFunc()
            defaultRectFunc()
            RenderSystem.disableBlend()
            RenderSystem.enableTexture()
        }
    }

    private companion object {
        const val inclination: Double = 2.5
    }
    override fun render(poseStack: PoseStack?) {
        runBlocking {
            RenderSystem.disableTexture()
            RenderSystem.enableBlend()
            RenderSystem.defaultBlendFunc()
            with(buffer) {
                begin(GL20.GL_POLYGON, DefaultVertexFormat.POSITION_COLOR)
                // Render the header:
                vertex(x, y + height, 0.0).colorEnd()
                vertex(x + width + inclination, y + height, 0.0).colorEnd()
                vertex(x + width, y, 0.0).colorEnd()
                vertex(x + inclination, y, 0.0).colorEnd()

                // Render the box:
                val currentHeight = (y + height) + 80.0
                vertex(x, currentHeight, 0.0).colorEnd()
                vertex(x + width + inclination, currentHeight, 0.0).colorEnd()
                vertex(x + width + inclination, y + height, 0.0).colorEnd()
                tessellator.end()

                begin(GL20.GL_LINES, DefaultVertexFormat.POSITION_COLOR)
                vertex(x + inclination, y + height, 0.0).colorEnd(0xF2F2FC)
                vertex(x + width, y + height, 0.0).colorEnd(0xF2F2FC)
                tessellator.end()
            }
            RenderSystem.enableTexture()
            RenderSystem.disableBlend()

            val renderButtons = if (buttons.isNotEmpty()) launch {
                scrollBar.apply {
                    x = this@ClickBox.x + this@ClickBox.width - 3
                    y = this@ClickBox.y + this@ClickBox.height + 3
                }.display(poseStack)
                buttons.forEachIndexed { num, it ->
                    it.apply {
                        it.x = this@ClickBox.x + 3
                        it.y = this@ClickBox.y + 5 + height + (num * 9)
                    }.display(poseStack)
                }
            } else null
            if ((name != null) && (poseStack != null)) {
                Minecraft.getInstance().font.draw(poseStack, name.string, x.toFloat() + 5, y.toFloat() + 1.5F, TransparentColor)
                renderButtons?.join()
            }
        }
    }
}